from __future__ import annotations
import asyncio
from kinopoisk.data.id import *
from dataclasses import dataclass
from dataclasses import field
from datetime import datetime



@dataclass(frozen=True)
class Review:
    id : Id = None
    is_positive : bool = None
    is_negative : bool = None
    date : datetime = None
    author : str = None
    title : str = None
    text : str = None
    user_positive_reviews_count : int = None
    user_negative_reviews_count : int = None


    @staticmethod
    async def _create_from_json(json : dict)-> TVSeries:
        review = Review(
            id=Id(
                json.get('reviewId'),
                None
            ),
            is_positive=True if json.get('reviewType') == 'POSITIVE' else False,
            is_negative=True if json.get('reviewType') == 'NEGATIVE' else False,
            date=datetime.strptime(json.get('reviewData'), '%Y-%m-%dT%H:%M:%S') if json.get('reviewData') is not None else None,
            author=json.get('reviewAutor'),
            title=json.get('reviewTitle'),
            text=json.get('reviewDescription'),
            user_positive_reviews_count=json.get('userPositiveRating'),
            user_negative_reviews_count=json.get('userNegativeRating')
        )
        return review



from dataclasses import dataclass
import enum


class ProdactionStatus(enum.Enum):
    filming = 'FILMING'
    preprodaction = 'PRE_PRODUCTION'
    completed = 'COMPLETED'
    announced = 'ANNOUNCED'
    unknow = 'UNKNOWN'
    postprodaction = 'POST_PRODUCTION'



from __future__ import annotations
import asyncio
from kinopoisk.data.movie.base_movie import BaseMovie
from kinopoisk.data.id import Id
from kinopoisk.data.name import Name
from kinopoisk.data.poster import Poster
from kinopoisk.data.raiting import Raiting, RaitingData
from kinopoisk.data.age_raiting import AgeRaiting
from kinopoisk.data.url import Url
from kinopoisk.data.description import Description
from dataclasses import dataclass
from dataclasses import field
from datetime import datetime
import kinopoisk.utils as utils


@dataclass(frozen=True)
class Film(BaseMovie):
    """Simple film dataclass"""
    
    id : Id = None
    name : Name = None
    poster : Poster = None
    raiting : Raiting = None
    url : Url = None
    year : int = None
    length : int = None
    slogan : str = None
    description : str = None
    editor_annotation : str = None
    is_tickets_available : bool = None
    prodaction_status : str = None
    age_rating : AgeRaiting = None
    has_imax : bool = None
    has_3d : bool = None
    last_sync : datetime = None
    countries : list[str] = field(default_factory=list)
    genres : list[str] = field(default_factory=list)


    @staticmethod
    async def _create_from_json(json : dict) -> Film:
        film = Film(
            id=Id(
                json.get('kinopoiskId') if json.get('kinopoiskId') is not None else json.get('filmId'),
                json.get('imdbId')),
            name=Name(original=json.get('nameOriginal'), en=json.get('nameEn'), ru=json.get('nameRu')),
            poster=Poster(json.get('posterUrl'), json.get('posterUrlPreview')),
            raiting=Raiting(
                good_review=RaitingData(json.get('ratingGoodReview'), json.get('ratingGoodReviewVoteCount')),
                kinopoisk=RaitingData(json.get('ratingKinopoisk') or json.get('rating'), json.get('ratingKinopoiskVoteCount') or json.get('ratingVoteCount')),
                imdb=RaitingData(json.get('ratingImdb'), json.get('ratingImdbVoteCount')),
                film_critics=RaitingData(json.get('ratingFilmCritics'), json.get('ratingFilmCriticsVoteCount')),
                await_=RaitingData(json.get('ratingAwait'), json.get('ratingAwaitCount')), 
                rf_critics=RaitingData(json.get('ratingRfCritics'), json.get('ratingRfCriticsVoteCount'))),
            url=Url(
                json.get('webUrl') if json.get('webUrl') is not None else f'https://www.kinopoisk.ru/film/{json.get("kinopoiskId") or json.get("filmId")}/',
                f'https://www.imdb.com/title/{json.get("imdbId")}/' if json.get("imdbId") is not None else None),
            year=json.get('year'),
            length=json.get('filmLength') if not isinstance(json.get('filmLength'), str) else sum(await utils.time_to_minute(json.get('filmLength'))),
            slogan=json.get('slogan'),
            description=Description(
                json.get('description'),
                json.get('shortDescription')),
            editor_annotation=json.get('editorAnnotation'),
            is_tickets_available=json.get('isTicketsAvailable'),
            prodaction_status=json.get('productionStatus'),
            age_rating=AgeRaiting(
                mpaa=json.get('ratingMpaa'),
                age_limit=int(json.get('ratingAgeLimits')[3:]) if json.get('ratingAgeLimits') is not None else None),
            has_imax=json.get('hasImax'),
            has_3d=json.get('has3D'),
            last_sync=datetime.strptime(json.get('lastSync'), '%Y-%m-%dT%H:%M:%S.%f') if json.get('lastSync') is not None else None,
            countries=([i['country'] for i in json.get('countries')] if json.get('countries') is not None else None),
            genres=([i['genre'] for i in json.get('genres')] if json.get('genres') is not None else None)
            )
        return film



from __future__ import annotations
import asyncio
from kinopoisk.data.id import Id
from kinopoisk.data.name import Name
from kinopoisk.data.poster import Poster
from kinopoisk.data.url import Url
from kinopoisk.data.types import ImageTypes
from dataclasses import dataclass
from dataclasses import field
import kinopoisk.utils as utils


@dataclass(frozen=True)
class BaseMovie:
    """Base class with general methods"""
    
    id : Id = None
    name : Name = None
    poster : Poster = None
    year : int = None
    length : int = None
    url : Url = None
    countries : list[str] = field(default_factory=list)
    genres : list[str] = field(default_factory=list)


    @staticmethod
    async def _create_from_json(json : dict) -> BaseMovie:
        movie = BaseMovie(
            id=Id(
                json.get('kinopoiskId') if json.get('kinopoiskId') is not None else json.get('filmId'),
                json.get('imdbId')),
            name=Name(original=json.get('nameOriginal'), en=json.get('nameEn'), ru=json.get('nameRu')),
            poster=Poster(json.get('posterUrl'), json.get('posterUrlPreview')),
            year=int(json.get('year')) if json.get('year') is not None else None,
            length=json.get('filmLength') if not isinstance(json.get('filmLength'), str) else sum(await utils.time_to_minute(json.get('filmLength'))),
            url=Url(
                json.get('webUrl') if json.get('webUrl') is not None else f'https://www.kinopoisk.ru/film/{json.get("kinopoiskId") or json.get("filmId")}/',
                f'https://www.imdb.com/title/{json.get("imdbId")}/' if json.get("imdbId") is not None else None),
            countries=([i['country'] for i in json.get('countries')] if json.get('countries') is not None else None),
            genres=([i['genre'] for i in json.get('genres')] if json.get('genres') is not None else None)
            ) 
        return movie


    async def get_all_data(self, client):
        """
        Movie may not have all data, this could be that movie may be take from various API methods, which have different return data details levels.
        @warning: Movie instance changes after call this method to needed movie class e.g. been BaseMovie became Film. 
        
        @param client: Client instance for delegating of methods.
        """

        new_ = await client.get_movie_data(self.id.kinopoisk)
        object.__setattr__(self, '__class__', new_.__class__)
        self.__dict__.update(new_.__dict__)


    async def get_reviews(self, client, page : int=None) -> (list[Review], None):
        """
        Gets reviews of this movie.
        
        @param client: Client instance for delegating of methods.
        """

        return await client.get_reviews(self.id.kinopoisk, page)
    

    async def get_persons(self, client) -> (list[Person], None):
        """
        Gets persons of this movie.
        
        @param client: Client instance for delegating of methods.
        """

        return await client.get_persons_of_movie(self.id.kinopoisk)
    

    async def get_similars(self, client) -> (list[BaseMovie], None):
        """
        Gets persons of this movie.
        
        @param client: Client instance for delegating of methods.
        """

        return await client.get_similars(self.id.kinopoisk)
    

    async def get_images(self, client, type : ImageTypes=ImageTypes.frame, page : int=None) -> (list[BaseMovie], None):
        """
        Gets images of this movie.
        
        @param client: Client instance for delegating of methods.
        """

        return await client.get_images(self.id.kinopoisk, type, page)


import asyncio
import kinopoisk.client

from fuzzywuzzy import fuzz
from fuzzywuzzy import process

async def time_to_minute(str) -> int:
    """Sometime API returning movie length as str e.g. \'1:46\'. So that this function just getting minuts from it string"""
    return (x * int(t) for x, t in zip([60, 1], str.split(':')))


async def get_genre_id_by_name(name : str, is_fzf : bool=True) -> (int, None):
    """
    Search by genre name in the list of genres and returns its id.

    Before used this func you shoulde be call load_filters method of any instance KPClient else this not be to work.
    
    :param name: Name of genre
    :param is_fzf: For fuzzy search.
    :return: Id of the genre. If some error or not found - None.
    """
    if kinopoisk.client.KPClient.filters is None: return
    
    name = name.lower()

    if not is_fzf:
        for i in kinopoisk.client.KPClient.filters['genres']:
            if name == i.name.lower(): return i.id
    else:
        for i in kinopoisk.client.KPClient.filters['genres']:
            if fuzz.ratio(name, i.name.lower()) >= 85: return i.id


async def get_country_id_by_name(name : str, is_fzf : bool=True) -> (int, None):
    """
    Search by country name in the list of countries and returns its id.

    Before used this func you shoulde be call load_filters method of any instance KPClient else this not be to work.
    
    :param name: Name of country
    :param is_fzf: For fuzzy search
    :return: Id of the country. If some error or not found - None.
    """
    if kinopoisk.client.KPClient.filters is None: return

    name = name.lower()

    if not is_fzf:
        for i in kinopoisk.client.KPClient.filters['countries']:
            if name == i.name.lower(): return i.id
    else:
        for i in kinopoisk.client.KPClient.filters['countries']:
            if fuzz.ratio(name, i.name.lower()) >= 85: return i.id




from __future__ import annotations
import asyncio
from dataclasses import dataclass



@dataclass(frozen=True)
class Fact:
    text : str = None
    is_spoiler : bool = None


    @staticmethod
    async def _create_from_json(json : dict)-> Fact:
        fact = Fact(
            text=json.get('text'),
            is_spoiler=False)
        return fact




from dataclasses import dataclass


@dataclass(frozen=True)
class RaitingData:
    value : float = 0
    count : int = 0


@dataclass(frozen=True)
class Raiting:
    good_review : RaitingData
    kinopoisk : RaitingData
    imdb : RaitingData
    film_critics : RaitingData
    await_ : RaitingData
    rf_critics : RaitingData



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['kinopoisk',
 'kinopoisk.data',
 'kinopoisk.data.filters',
 'kinopoisk.data.movie',
 'kinopoisk.data.movie.tv_series',
 'kinopoisk.errors',
 'kinopoisk.utils']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.8.1,<4.0.0',
 'fuzzywuzzy>=0.18.0,<0.19.0',
 'python-Levenshtein>=0.12.2,<0.13.0']

setup_kwargs = {
    'name': 'kinopoisk-unofficial-api',
    'version': '0.7.0',
    'description': 'Simple wrapper of kinopoiskapiunofficial.tech',
    'long_description': '# kinopoisk unofficial API\n\nThis is simple python package for getting data from [unofficial kinopoisk API](https://kinopoiskapiunofficial.tech).\n\n## Installing\n\n**pip**\n\n```bash\npip install kinopoisk-unofficial-api\n```\n\n**poetry**\n\n```bash\npoetry add kinopoisk-unofficial-api\n```\n\n## Getting token\n\nWhy this not work. What the token and why this require it from me?\nFor interact to [API](https://kinopoiskapiunofficial.tech) you should getting api token. That get it you need sign up to [their site](https://kinopoiskapiunofficial.tech/signup). After register go to profile and save your token somewhere.\n\n## How to use\n\nFor begin you should create the **KPClient** instance.\n\n```python\nfrom kinopoisk import KPClient\n\nclient = KPClient(<your token>)\n```\n\nWhen you have client you can used all functional this library.\n\n### Getting movie\n\n```python\nmatrix = await client.get_movie_data(301)\nprint(matrix)\n```\n\nYou can get e.g. name, release date, raiting, length of this movie and more.\n\n```python\nmatrix.name.en\n\'The Matrix\'\nmatrix.year\n1999\nmatrix.length\n136\n```\n\nIf you not know movie id (that to be often) may use another method named ***search_movie***\n\n```python\nanswer = await client.search_movie(\'Mr. Robot\')\nmr_robot = answer[0] # If you search popular movie, that usually this movie should be to first\n```\n\n### Getting data of movie\nIn previous example we got tv series. By default it take without it seasons. That load it you should get it id and call to method of client  ***get_seasons_data***\n\n```python\nseasons = await client.get_seasons_data(mr_robot.id.kinopoisk)\nfor season in seasons:\n\tprint(season.episodes)\n```\n\nYet this not exactly conveniently. Store seasons and it tv series between it may be not good idea. So that, you may not splitting data, for it need call tv series method ***load_seasons***.\n\n```python\nawait mr_robot.load_seasons(client)\nfor season in mr_robot.seasons:\n    print(season.episodes)\n# Or just\nfor season in mr_robot:\n    print(season.episodes)\n```\n\nSeason have a episodes (Seriously?) that may be get it same way.\n\n```python\nfor season in mr_robot:\n    for episode in season:\n        print(episode.name.en)\n\t\t# First episode named \'eps1.0_hellofriend.mov\'\n```\n\n### Getting facts and bloopers of movie\n\n```python\nfor fact in await client.get_facts(mr_robot.id.kinopoisk):\n\tprint(fact.text)\n```\n\n### Getting persons\n\n```python\nbc = (await client.search_person(\'Benedict Cumberbatch\'))[0]\nawait bc.get_all_data(client)\nprint(bc.birthday.strftime("%d %B, %Y"))\n# Output 19 July, 1976\n```\n\nOr you can get persons of some movie\n\n```python\npersons = await mr_robot.get_persons(client)\nactors = []\nfor person in persons:\n    if person.is_actor:\n        actors.append(person)\nfor actor in actors[:10]: print(f\'{actor.name.en}: {actor.character}\')\n```\n\n### Getting reviews\n\n```python\nreviews = await mr_robot.get_reviews(client)\nfor review in reviews: print(f\'{review.author} - {review.title}:\\n{review.text}\')\n```\n\n### Getting similars movies\n\n```python\nmovies = await mr_robot.get_similars(client)\nfor movie in movies:\n    print(movie.name.en)\n```\n\n```\nFight Club\nWho Am I - Kein System ist sicher\nThe Matrix\nDexter\nA Beautiful Mind\nHackers\nThe Social Network\nThe Fifth Estate\nV for Vendetta\nBlack Mirror\n23\n```\n\n### Getting images\n\nYou can get different images e.g. posters wallpapers, backstage photo and more\n\n```python\nimages = await mr_robot.get_images(client, ImageTypes.poster)\nfor image in images:\n    print(image.big)\n```\n\n<p style="display: flex;">\n\t<img height=250 src="https://avatars.mds.yandex.net/get-kinopoisk-image/1704946/981bdebd-d27d-4ea4-85eb-9c51c0bd678b/orig" />\n\t<img height=250 src="https://avatars.mds.yandex.net/get-kinopoisk-image/1704946/cc2adcad-a448-42b1-a329-c6c222b047af/orig" /> \n\t<img height=250 src="https://avatars.mds.yandex.net/get-kinopoisk-image/1900788/f69aae71-cb44-432e-aaf5-657e551b018d/orig" />\n\t<img height=250 src="https://avatars.mds.yandex.net/get-kinopoisk-image/1898899/49853c24-b2b2-4698-9b3a-74e6ef0e37b6/orig">\n</p>\n### Getting some tops\n\n**Best 250**\n\n```python\nfor movie in (await client.get_top(TopTypes.best_250))[:5]:\n\tprint(movie.name.en)\n```\n\n**Popular 100**\n\n```python\nfor movie in (await client.get_top(TopTypes.popular_100))[:5]:\n\tprint(movie.name.en)\n```\n\n**Future**\n\n```python\nfor movie in (await client.get_top(TopTypes.best_250))[:5]:\n\tprint(movie.name.en)\n```\n\n',
    'author': 'Nawot',
    'author_email': 'Nawot001@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Nawot/kinopoisk-unofficial-api',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



import enum


class MovieTypes(enum.Enum):

    def __str__(self):
        return self.value

    film = 'FILM'
    video = 'VIDEO'
    tv_series = 'TV_SERIES'
    mini_series = 'MINI_SERIES'
    tv_show = 'TV_SHOW'
    all = 'ALL'


class FactTypes(enum.Enum):
    
    def __str__(self):
        return self.value

    fact = 'FACT'
    blooper = 'BLOOPER'


class ImageTypes(enum.Enum):

    def __str__(self):
        return self.value


    frame = 'STILL'
    back_stage = 'SHOOTING'
    poster = 'POSTER'
    fan_art = 'FAN_ART'
    promo = 'PROMO'
    concept_art = 'CONCEPT'
    wallpaper = 'WALLPAPER'
    cover = 'COVER'
    screenshot = 'SCREENSHOT'


class TopTypes(enum.Enum):

    def __str__(self):
        return self.value


    best_250 = 'TOP_250_BEST_FILMS'
    popular_100 = 'TOP_100_POPULAR_FILMS'
    future = 'TOP_AWAIT_FILMS'